#include <stdio.h> 
#include <p18f452.h>
#include <delays.h>
#include "MYi2c.h"

//initialise the MSSP module
void InitI2C()
	{
	TRISBbits.TRISB0=1;
	TRISBbits.TRISB1=1;
	SSPCON1=0b00101000;
	SSPCON2=0;
	SSPSTAT=0b10000000;
	SSPADD=50;  //20000/(4*100);   //Fosc/(4*baud)
	PIR1bits.SSPIF=0;
	PIR2bits.BCLIF=0;
	}

//Wait for the module to finish it's last action.
void WaitSSP()
	{
	while(PIR1bits.SSPIF==0);
	PIR1bits.SSPIF=0;
	}

//send start bit
void SendStart()
	{
	SSPCON2bits.SEN=1;
	WaitSSP();
	}

//Send stop bit.
void SendStop()
	{
	SSPCON2bits.PEN=1;
	WaitSSP();
	}

//Send restart bit
void SendRestart()
	{
	SSPCON2bits.RSEN=1;
	WaitSSP();
	}

//Send byte and return ack bit - 1=Ack  0=NAck
char SendByte(char Byte)
	{
	SSPBUF=Byte;
	WaitSSP();
	return(!SSPCON2bits.ACKSTAT);
	}

//Get a byte from the slave
char ReceiveByte()
	{
	SSPCON2bits.RCEN=1;
	WaitSSP();
	return(SSPBUF);
	}

//Send a Not Acknowledge (NAck) to the slave
void SendNack()
	{
	SSPCON2bits.ACKDT=1;
	SSPCON2bits.ACKEN=1;
	WaitSSP();
	}

//Send an Acknowledge (Ack) to the slave
void SendAck()
	{
	SSPCON2bits.ACKDT=0;
	SSPCON2bits.ACKEN=1;
	WaitSSP();
	}

//Sends the start and device address.
//If the device is busy then it resends until accepted.
void SendID(char DeviceID)
	{
	SendStart();
	if(SendByte(DeviceID)==1)
		return;
	do
		{
		SendRestart();
		}while(SendByte(DeviceID)==0);
	}

//Write a byte to the EEPROM
char WriteByte(long Address, char Byte)
	{
	unsigned char addr = 0xa0;
	if(Address > 0xffff) addr += 8;
	SendID(addr);
	if(SendByte((int)Address>>8)==0)
		return(0);
	if(SendByte((int)Address&0xff)==0)
		return(0);
	if(SendByte(Byte)==0)
		return(0);
	SendStop();
	return(1);
	}

//Read a byte from the EEPROM
char ReadByte(long Address)
	{
	char Byte;
	unsigned char  addr = 0xa0;
	if(Address > 0xffff) addr += 8;
	SendID(addr);
	if(SendByte((int)Address>>8)==0)
		return(0);
	if(SendByte((int)Address&0xff)==0)
		return(0);
	SendRestart();
	if(SendByte(addr+1)==0)
		return(0);
	Byte=ReceiveByte();
	SendNack();
	SendStop();
	return(Byte);
	}
	
//Write a byte to The RTC
char WriteRTC(char Address, char Byte)
	{
	SendID(0b11010000);
	if(SendByte(Address&0xff)==0)
		return(0);
	if(SendByte(Byte)==0)
		return(0);
	SendStop();
	return(1);
	}

//Read a byte from the RTC
char ReadRTC(char Address)
	{
	char Byte;
	SendID(0b11010000);
	if(SendByte(Address&0xff)==0)
		return(0);
	SendRestart();
	if(SendByte(0b11010001)==0)
		return(0);
	Byte=ReceiveByte();
	SendNack();
	SendStop();
	return(Byte);
	}

void WriteClock(char *array)
	{
	char x;
	for(x=0;x<8;x++)
		WriteRTC(x,array[x]);	
	}	

void ReadClock(char *array)
	{
	char x;
	for(x=0;x<8;x++)
		array[x] = ReadRTC(x);	
	}	
unsigned int ReadInt(long addr)
	{
	unsigned int res;
	res = (unsigned char)ReadByte(addr);
	res = res<<8;
	res += (unsigned char)ReadByte(addr+1);
	return res;
	}

long ReadLong(long addr)
	{
	long res3;
	res3 = (unsigned int)ReadInt(addr);
	res3 = res3<<16;
	res3 +=	(unsigned int)ReadInt(addr+2);
	return 	res3;
	}  

void WriteInt(long addr, int val)
	{
	WriteByte(addr,val>>8);
	WriteByte(addr+1,val&0xff);
	}

void WriteLong(long addr, long val)
	{
	WriteInt(addr,val>>16);
	WriteInt(addr+2,val&0xffff);
	}  

void I2cReadStr(char *buff,long addr,int count)
	{
	while(count--)
		{
		*buff++ = ReadByte(addr++);
		}
	*buff = 0;
	}

void I2cWriteStr(char *buff,long addr,int count)
	{
	while(count--)
		{
		WriteByte(addr++,*buff++);
		}
	WriteByte(addr, 0);
	}
